# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import onfido.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0003_auto_20161019_1038'),
    ]

    operations = [
        migrations.AlterField(
            model_name='applicant',
            name='raw',
            field=onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='check',
            name='raw',
            field=onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='report',
            name='raw',
            field=onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import mock

from dateutil.parser import parse as date_parse

from django.contrib.auth.models import User
from django.db.models import Model
from django.test import TestCase

from ..models import (
    BaseModel,
    BaseStatusModel,
    Applicant,
    Check,
    Report,
)


class TestBaseModel(BaseModel):

    class Meta:
        managed = False


class TestBaseStatusModel(BaseStatusModel):

    class Meta:
        managed = False


class BaseModelTests(TestCase):

    def test_defaults(self):
        obj = TestBaseModel()
        self.assertEqual(obj.id, '')
        self.assertEqual(obj.created_at, None)
        self.assertEqual(obj.raw, {})

    @mock.patch.object(BaseModel, 'full_clean')
    @mock.patch.object(Model, 'save')
    def test_save(self, mock_save, mock_clean):
        """Test that save method returns self."""
        obj = TestBaseModel()
        self.assertEqual(obj.save(), obj)

    def test_parse(self):
        """Test the parse method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        obj = TestBaseModel().parse(data)
        self.assertEqual(obj.id, data['id'])
        self.assertEqual(obj.created_at, date_parse(data['created_at']))


class BaseStatusModelTests(TestCase):

    def test_defaults(self):
        obj = TestBaseStatusModel()
        self.assertEqual(obj.id, '')
        self.assertEqual(obj.created_at, None)
        self.assertEqual(obj.raw, {})
        self.assertEqual(obj.status, None)
        self.assertEqual(obj.result, None)
        self.assertEqual(obj.updated_at, None)
        self.assertEqual(obj.raw, {})

    def test_parse(self):
        """Test the parse method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        obj = TestBaseStatusModel().parse(data)
        self.assertEqual(obj.id, data['id'])
        self.assertEqual(obj.created_at, date_parse(data['created_at']))
        self.assertEqual(obj.status, data['status'])
        self.assertEqual(obj.result, data['result'])

    @mock.patch('onfido.signals.on_status_change.send')
    @mock.patch('onfido.signals.on_completion.send')
    @mock.patch.object(BaseStatusModel, 'save')
    def test_update_status(self, mock_save, mock_complete, mock_update):
        """Test the update_status method."""
        obj = TestBaseStatusModel(status='before')
        now = datetime.datetime.now()
        self.assertEqual(obj.status, 'before')
        self.assertEqual(obj.updated_at, None)

        # try passing in something that is not a datetime
        self.assertRaises(
            AssertionError,
            obj.update_status,
            'report.completed',
            'after',
            'now'
        )

        obj = obj.update_status('report.completed', 'after', now)
        self.assertEqual(obj.status, 'after')
        self.assertEqual(obj.updated_at, now)
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            event='report.completed',
            status_before='before',
            status_after='after'
        )
        mock_complete.assert_not_called()

        # if we send 'complete' as the status we should fire
        # the second signal
        mock_update.reset_mock()
        obj = obj.update_status('report.completed', 'complete', now)
        self.assertEqual(obj.status, 'complete')
        self.assertEqual(obj.updated_at, now)
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            event='report.completed',
            status_before='after',
            status_after='complete'
        )
        mock_complete.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj
        )


class ApplicantManagerTests(TestCase):

    """ApplicantManager tests."""
    TEST_DATA = {
        "id": "14d2335e-4586-4ac4-aecd-b18296e7d675",
        "created_at": "2016-10-15T19:05:07Z",
    }

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)

    @mock.patch.object(BaseModel, 'full_clean')
    def test_create_applicant(self, mock_clean):
        """Test the create method parses response."""
        data = ApplicantManagerTests.TEST_DATA
        applicant = Applicant.objects.create_applicant(user=self.user, raw=data)
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.raw, data)
        self.assertEqual(applicant.id, data['id'])
        self.assertEqual(applicant.created_at, date_parse(data['created_at']))


class ApplicantTests(TestCase):

    """Applicant models tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)

    def test_defaults(self):
        """Test default property values."""
        applicant = self.applicant
        self.assertEqual(applicant.id, 'foo')
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.created_at, None)

    def test_save(self):
        """Test the save method."""
        self.user.save()
        applicant = self.applicant.save()
        self.assertEqual(applicant.id, 'foo')
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.created_at, None)
        # test the related_name
        self.assertEqual(self.user.onfido_applicant, applicant)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        applicant = self.applicant
        self.assertIsNotNone(str(applicant))
        self.assertIsNotNone(unicode(applicant))
        self.assertIsNotNone(repr(applicant))


class CheckManagerTests(TestCase):

    """ApplicantManager tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)

    @mock.patch.object(BaseModel, 'full_clean')
    def test_create_check(self, mock_clean):
        """Test the create method parses response."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        check = Check.objects.create_check(
            applicant=self.applicant,
            raw=data
        )
        self.assertEqual(check.user, self.user)
        self.assertEqual(check.applicant, self.applicant)
        self.assertEqual(check.id, data['id'])
        self.assertEqual(check.check_type, data['type'])
        self.assertEqual(check.status, data['status'])
        self.assertEqual(check.result, data['result'])
        self.assertEqual(check.created_at, date_parse(data['created_at']))


class CheckTests(TestCase):

    """Check models tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)
        self.check = Check(id='bar', user=self.user, applicant=self.applicant)  # noqa

    def test_defaults(self):
        """Test default property values."""
        check = Check()
        self.assertEqual(check.id, '')
        self.assertEqual(check.created_at, None)
        self.assertEqual(check.status, None)
        self.assertEqual(check.result, None)
        self.assertEqual(check.check_type, '')

    def test_save(self):
        """Test save method."""
        self.user.save()
        self.applicant.save()
        self.check.check_type = 'standard'
        check = self.check.save()
        self.assertEqual(check.id, 'bar')
        self.assertEqual(check.created_at, None)
        self.assertEqual(check.status, None)
        self.assertEqual(check.result, None)
        self.assertEqual(check.check_type, 'standard')
        # test the related_names
        self.assertEqual(self.applicant.checks.get(), check)
        self.assertEqual(self.user.onfido_checks.get(), check)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        check = self.check
        self.assertIsNotNone(str(check))
        self.assertIsNotNone(unicode(check))
        self.assertIsNotNone(repr(check))

    def test_parse(self):
        """Test the parse_raw method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        check = Check()
        check.parse(data)
        # real data taken from check.json
        self.assertEqual(check.id, "c26f22d5-4903-401f-8a48-7b0211d03c1f")
        self.assertEqual(check.created_at, date_parse(data['created_at']))
        self.assertEqual(check.status, "awaiting_applicant")
        self.assertEqual(check.result, "clear")


class ReportManagerTests(TestCase):

    """ReportManager tests."""
    TEST_DATA = {
        "created_at": "2016-10-18T16:02:08Z",
        "id": "1ffd3e8a-da71-4674-a245-8b52f1492191",
        "name": "identity",
        "result": "clear",
        "status": "awaiting_data",
        "variant": "standard"
    }

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)
        self.check = Check(user=self.user, applicant=self.applicant, check_type='standard')

    @mock.patch.object(BaseModel, 'full_clean')
    def test_create_report(self, mock_clean):
        """Test the create method parses response."""
        data = ReportManagerTests.TEST_DATA
        report = Report.objects.create_report(
            check=self.check,
            raw=data
        )
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.id, data['id'])
        self.assertEqual(report.report_type, data['name'])
        self.assertEqual(report.status, data['status'])
        self.assertEqual(report.result, data['result'])
        self.assertEqual(report.created_at, date_parse(data['created_at']))


class ReportTests(TestCase):

    """Report models tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(id='foo', user=self.user)
        self.check = Check(id='bar', user=self.user, applicant=self.applicant, check_type='standard')  # noqa
        self.report = Report(id='baz', user=self.user, onfido_check=self.check)  # noqa

    def test_defaults(self):
        """Test default property values."""
        report = self.report
        self.assertEqual(report.id, 'baz')
        self.assertEqual(report.created_at, None)
        self.assertEqual(report.status, None)
        self.assertEqual(report.result, None)
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.report_type, '')

    def test_save(self):
        """Test save method."""
        self.user.save()
        self.applicant.save()
        self.check.save()
        self.report.report_type = 'identity'
        report = self.report.save()
        self.assertEqual(report.id, 'baz')
        self.assertEqual(report.created_at, None)
        self.assertEqual(report.status, None)
        self.assertEqual(report.result, None)
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.report_type, 'identity')
        # test the related_names
        self.assertEqual(self.check.reports.get(), report)
        self.assertEqual(self.user.onfido_reports.get(), report)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        report = self.report
        self.assertIsNotNone(str(report))
        self.assertIsNotNone(unicode(report))
        self.assertIsNotNone(repr(report))

    def test_parse(self):
        """Test the parse_raw method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "result": "clear",
            "name": "identity",
        }
        report = Report()
        report.parse(data)
        # real data taken from check.json
        self.assertEqual(report.id, "c26f22d5-4903-401f-8a48-7b0211d03c1f")
        self.assertEqual(report.created_at, date_parse(data['created_at']))
        self.assertEqual(report.status, "awaiting_applicant")
        self.assertEqual(report.result, "clear")
        self.assertEqual(report.report_type, "identity")



# -*- coding: utf-8 -*-
from decimal import Decimal

from django.test import TestCase

from ..admin import (
    RawMixin,
    Applicant
)


class RawMixinTests(TestCase):

    def test__raw(self):
        mixin = RawMixin()
        obj = Applicant(raw={'foo': 'bar'})
        html = mixin._raw(obj)
        self.assertEqual(html, '<code>{<br>&nbsp;&nbsp;&nbsp;&nbsp;"foo":&nbsp;"bar"<br>}</code>')

        # test with Decimal (stdlib json won't work) and unicode
        obj = Applicant(raw={'foo': Decimal(1.0), 'bar': u'åß∂ƒ©˙∆'})
        html = mixin._raw(obj)



# -*- coding: utf-8 -*-
import mock

from dateutil.parser import parse as date_parse

from django.contrib.auth.models import User
from django.test import TestCase

from ..api import (
    _get,
    _post,
    _headers,
    _respond,
    ApiError,
    create_applicant,
    create_check,
    # import from api, not models, to reduce dependency issues
    Applicant,
    Check,
    Report
)


class ApiTests(TestCase):

    """Api module tests (not testing the API itself)."""

    def test__headers(self):
        """Test the _headers function return valid dict."""
        self.assertEqual(
            _headers(api_key="foo"),
            {
                "Authorization": "Token token=foo",
                "Content-Type": "application/json"
            }
        )

    def test__respond(self):
        """Test the _respond function handles 2xx."""
        response = mock.Mock()
        response.status_code = 200
        self.assertEqual(
            _respond(response),
            response.json.return_value
        )
        # non-2xx should raise error
        response.status_code = 400
        response.json.return_value = {
            "error": {
                "message": "foo",
                "type": "bar"
            }
        }
        self.assertRaises(ApiError, _respond, response)

    @mock.patch('requests.get')
    @mock.patch('onfido.api._headers')
    def test_get(self, mock_headers, mock_get):
        """Test the get function calls API."""
        response = mock.Mock()
        response.status_code = 200
        headers = mock_headers.return_value
        mock_get.return_value = response
        self.assertEqual(_get('/'), response.json.return_value)
        mock_get.assert_called_once_with('/', headers=headers)

    @mock.patch('requests.post')
    @mock.patch('onfido.api._headers')
    def test_post(self, mock_headers, mock_post):
        """Test the get function calls API."""
        response = mock.Mock()
        response.status_code = 200
        headers = mock_headers.return_value
        data = {"foo": "bar"}
        mock_post.return_value = response
        self.assertEqual(_post('/', data), response.json.return_value)
        mock_post.assert_called_once_with('/', headers=headers, json=data)

    @mock.patch('onfido.api._post')
    def test_create_applicant(self, mock_post):
        """Test the create_applicant function."""
        data = {
            "id": "a9acefdf-3dc5-4973-aa78-20bd36825b50",
            "created_at": "2016-10-18T16:02:04Z",
            "title": "Mr",
            "first_name": "Fred",
            "last_name": "Flintstone",
            "email": "fred@flintstone.com",
            "country": "gbr",
        }
        mock_post.return_value = data
        user = User.objects.create_user(
            username='fred',
            first_name='Fred',
            last_name='Flintstone',
            email='fred@flintstone.com'
        )
        applicant = create_applicant(user=user)
        self.assertEqual(applicant.id, data['id'])
        self.assertEqual(applicant.user, user)
        self.assertEqual(applicant.created_at, date_parse(data['created_at']))

    @mock.patch('onfido.api._post')
    def test_create_check(self, mock_post):
        """Test the create_check function."""
        applicant_data = {
            "id": "a9acefdf-3dc5-4973-aa78-20bd36825b50",
            "created_at": "2016-10-18T16:02:04Z",
            "title": "Mr",
            "first_name": "Fred",
            "last_name": "Flintstone",
            "email": "fred@flintstone.com",
            "country": "gbr",
        }
        check_data = {
            "id": "b2b75f66-fffd-45a4-ba15-b1e77a672a9a",
            "created_at": "2016-10-18T16:02:08Z",
            "status": "awaiting_applicant",
            "redirect_uri": None,
            "type": "standard",
            "result": None,
            # "form_uri": "https://onfido.com/information/b2b75f66-fffd-45a4-ba15-b1e77a672a9a",
            "reports": [
                {
                    "created_at": "2016-10-18T16:02:08Z",
                    "id": "08345559-852c-4f47-bf65-f9852bf59c4b",
                    "name": "document",
                    "result": None,
                    "status": "awaiting_data",
                    "variant": "standard",
                },
                {
                    "created_at": "2016-10-18T16:02:08Z",
                    "id": "1ffd3e8a-da71-4674-a245-8b52f1492191",
                    "name": "identity",
                    "result": None,
                    "status": "awaiting_data",
                    "variant": "standard",
                }
            ],
        }
        mock_post.return_value = check_data
        user = User.objects.create_user(
            username='fred',
            first_name='Fred',
            last_name='Flintstone',
            email='fred@flintstone.com'
        )
        applicant = Applicant.objects.create_applicant(user, applicant_data)

        # 1. use the defaults.
        check = create_check(
            applicant=applicant,
            check_type='standard',
            reports=['identity', 'document']
        )
        self.assertEqual(Check.objects.get(), check)
        # check we have two reports, and that the raw field matches the JSON
        # and that the parse method has run
        self.assertEqual(Report.objects.count(), 2)
        for r in check_data['reports']:
            # this will only work if the JSON has been parsed correctly
            report = Report.objects.get(id=r['id'])
            self.assertEqual(report.raw, r)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='applicant',
            name='created_at',
            field=models.DateTimeField(help_text='The timestamp returned from the Onfido API.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='check',
            name='created_at',
            field=models.DateTimeField(help_text='The timestamp returned from the Onfido API.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='check',
            name='result',
            field=models.CharField(blank=True, max_length=20, null=True, help_text='The final result of the check / reports (from API).', choices=[(b'Check', ((b'clear', b'Clear'), (b'consider', b'Consider'))), (b'Report', ((b'clear', b'Clear'), (b'consider', b'Consider'), (b'unidentified', b'Unidentified')))]),
        ),
        migrations.AlterField(
            model_name='check',
            name='status',
            field=models.CharField(blank=True, max_length=20, null=True, help_text='The current state of the check / report (from API).', choices=[(b'Check', ((b'in_progress', b'In progress'), (b'awaiting_applicant', b'Awaiting applicant'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'reopened', b'Reopened'))), (b'Report', ((b'awaiting_data', b'Awaiting data'), (b'awaiting_approval', b'Awaiting approval'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'cancelled', b'Cancelled')))]),
        ),
        migrations.AlterField(
            model_name='check',
            name='updated_at',
            field=models.DateTimeField(help_text='The timestamp of the most recent status change (from API).', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='report',
            name='created_at',
            field=models.DateTimeField(help_text='The timestamp returned from the Onfido API.', null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='report',
            name='report_type',
            field=models.CharField(help_text='The name of the report - see https://documentation.onfido.com/#reports', max_length=20, choices=[(b'identity', b'Identity report'), (b'document', b'Document report'), (b'street_level', b'Street level report'), (b'facial_similarity', b'Facial similarity report')]),
        ),
        migrations.AlterField(
            model_name='report',
            name='result',
            field=models.CharField(blank=True, max_length=20, null=True, help_text='The final result of the check / reports (from API).', choices=[(b'Check', ((b'clear', b'Clear'), (b'consider', b'Consider'))), (b'Report', ((b'clear', b'Clear'), (b'consider', b'Consider'), (b'unidentified', b'Unidentified')))]),
        ),
        migrations.AlterField(
            model_name='report',
            name='status',
            field=models.CharField(blank=True, max_length=20, null=True, help_text='The current state of the check / report (from API).', choices=[(b'Check', ((b'in_progress', b'In progress'), (b'awaiting_applicant', b'Awaiting applicant'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'reopened', b'Reopened'))), (b'Report', ((b'awaiting_data', b'Awaiting data'), (b'awaiting_approval', b'Awaiting approval'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'cancelled', b'Cancelled')))]),
        ),
        migrations.AlterField(
            model_name='report',
            name='updated_at',
            field=models.DateTimeField(help_text='The timestamp of the most recent status change (from API).', null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Basic wire operations with the API - GET/POST/PUT.

This is a simple wrapper around requests.

"""
import logging
import urlparse

import requests

from .models import (
    Applicant,
    Check,
    Report
)
from .settings import (
    ONFIDO_API_ROOT,
    ONFIDO_API_KEY
)

logger = logging.getLogger(__name__)


class ApiError(Exception):

    """Error raised when interacting with the API."""

    def __init__(self, response):
        """Initialise error from response object."""
        data = response.json()
        logger.debug("Onfido API error: {}".format(data))
        super(ApiError, self).__init__(data['error']['message'])
        self.error_type = data['error']['type']


def _url(path):
    """Format absolute API URL."""
    return urlparse.urljoin(ONFIDO_API_ROOT, path)


def _headers(api_key=ONFIDO_API_KEY):
    """Format request headers."""
    return {
        "Authorization": "Token token={0}".format(api_key),
        "Content-Type": "application/json"
    }


def _respond(response):
    """Process common response object."""
    logger.debug(response)
    if not str(response.status_code).startswith('2'):
        raise ApiError(response)
    return response.json()


def _get(url):
    """Make a GET request and return the response as JSON."""
    logger.debug("Making GET request to %s", url)
    return _respond(requests.get(url, headers=_headers()))


def _post(url, data):
    """Make a POST request and return the response as JSON."""
    logger.debug("Making POST request to %s", url)
    return _respond(requests.post(url, headers=_headers(), json=data))


def create_applicant(user):
    """Create an applicant in the Onfido system."""
    data = {
        "first_name": user.first_name,
        "last_name": user.last_name,
        "email": user.email
    }
    response = _post(_url('applicants'), data)
    logger.debug(response)
    return Applicant.objects.create_applicant(user, response)


def create_check(
    applicant, check_type, reports,
    suppress_form_emails=True, redirect_uri=None
):
    """
    Create a new Check (and child Reports).

    Args:
        applicant: Applicant for whom the checks are being made.
        check_type: string, currently only 'standard' is supported.
        reports: list of strings, each of which is a valid report type.

    Kwargs:
        suppress_form_emails: bool, if True then suppress the email the Onfido
            itself would normally send when a check is initiated. Defaults to
            True.
        redirect_uri: string, a url to which to direct the user _after_ they
            have submitted all the information requested.

    Returns a new Check object, and creates the child Report objects.

    """
    data = {
        "type": check_type,
        "suppress_form_emails": suppress_form_emails,
        "reports": [{'name': r for r in reports}],
        "redirect_uri": redirect_uri
    }
    response = _post(_url('applicants/{}/checks'.format(applicant.id)), data)
    logger.debug(response)
    check = Check.objects.create_check(applicant=applicant, raw=response)
    for report in response['reports']:
        Report.objects.create_report(check=check, raw=report)
    return check



# -*- coding: utf-8 -*-
import simplejson as json  # simplejson supports Decimal

from django.contrib import admin
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from .models import (
    Applicant,
    Check,
    Report
)


class RawMixin(object):

    """Admin mixin used to pprint raw JSON fields."""

    def _raw(self, obj):
        """
        Return an indented HTML pretty-print version of JSON.

        Take the event_payload JSON, indent it, order the keys and then
        present it as a <code> block. That's about as good as we can get
        until someone builds a custom syntax function.

        """
        pretty = json.dumps(
            obj.raw,
            sort_keys=True,
            indent=4,
            separators=(',', ': ')
        )
        html = pretty.replace(" ", "&nbsp;").replace("\n", "<br>")
        return mark_safe("<code>{}</code>".format(html))
    _raw.short_description = _("Raw (from API)")


class ApplicantAdmin(RawMixin, admin.ModelAdmin):

    """Admin model for Applicant objects."""

    list_display = ('id', 'user', 'created_at')
    list_filter = ('created_at',)
    ordering = ('user__first_name', 'user__last_name')
    readonly_fields = ('id', 'created_at', '_raw')
    raw_id_fields = ('user',)
    exclude = ('raw',)

admin.site.register(Applicant, ApplicantAdmin)


class CheckAdmin(RawMixin, admin.ModelAdmin):

    """Admin model for Check objects."""

    list_display = (
        'id', 'applicant', 'check_type', 'status',
        'result', 'created_at', 'updated_at'
    )
    readonly_fields = (
        'id', 'created_at', 'applicant', 'check_type',
        'status', 'result', 'updated_at', '_raw'
    )
    list_filter = ('check_type', 'created_at',)
    raw_id_fields = ('applicant', 'user')
    exclude = ('raw',)

admin.site.register(Check, CheckAdmin)


class ReportAdmin(RawMixin, admin.ModelAdmin):

    """Admin model for Report objects."""

    list_display = (
        'onfido_check', 'report_type', 'status',
        'result', 'created_at', 'updated_at'
    )
    readonly_fields = (
        'id', 'user', 'onfido_check', 'report_type',
        'status', 'result', 'created_at', 'updated_at', '_raw'
    )
    list_filter = ('created_at', 'report_type')
    raw_id_fields = ('onfido_check', 'user')
    exclude = ('raw',)

admin.site.register(Report, ReportAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0002_auto_20161019_1037'),
    ]

    operations = [
        migrations.AlterField(
            model_name='applicant',
            name='user',
            field=models.OneToOneField(related_name='onfido_applicant', to=settings.AUTH_USER_MODEL, help_text='Django user that maps to this applicant.'),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import logging

from dateutil.parser import parse as date_parse

from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext as _

from .db.fields import JSONField
from .signals import on_status_change, on_completion

logger = logging.getLogger(__name__)


class BaseModel(models.Model):

    """Base model used to set timestamps."""

    id = models.CharField(
        primary_key=True,
        max_length=40,
        help_text=_("The id returned from the Onfido API."),
    )
    created_at = models.DateTimeField(
        help_text=_("The timestamp returned from the Onfido API."),
        blank=True, null=True
    )
    raw = JSONField(
        help_text=_("The raw JSON returned from the API."),
        blank=True, null=True
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        """Save object and return self (for chaining methods)."""
        self.full_clean()
        super(BaseModel, self).save(*args, **kwargs)
        return self

    def parse(self, raw_json):
        """Parses the raw value out into other properties."""
        self.raw = raw_json
        self.id = self.raw['id']
        self.created_at = date_parse(self.raw['created_at'])
        return self


class BaseStatusModel(BaseModel):

    """Base class for models with a status and result field."""

    CHECK_STATUS_CHOICES = (
        ('in_progress', 'In progress'),
        ('awaiting_applicant', 'Awaiting applicant'),
        ('complete', 'Complete'),
        ('withdrawn', 'Withdrawn'),
        ('paused', 'Paused'),
        ('reopened', 'Reopened'),
    )
    REPORT_STATUS_CHOICES = (
        ('awaiting_data', 'Awaiting data'),
        ('awaiting_approval', 'Awaiting approval'),
        ('complete', 'Complete'),
        ('withdrawn', 'Withdrawn'),
        ('paused', 'Paused'),
        ('cancelled', 'Cancelled'),
    )
    STATUS_CHOICES = (
        ('Check', CHECK_STATUS_CHOICES),
        ('Report', REPORT_STATUS_CHOICES)
    )
    CHECK_RESULT_CHOICES = (
        ('clear', 'Clear'),
        ('consider', 'Consider')
    )
    REPORT_RESULT_CHOICES = (
        ('clear', 'Clear'),
        ('consider', 'Consider'),
        ('unidentified', 'Unidentified'),
    )
    RESULT_CHOICES = (
        ('Check', CHECK_RESULT_CHOICES),
        ('Report', REPORT_RESULT_CHOICES),
    )

    status = models.CharField(
        max_length=20,
        help_text=_("The current state of the check / report (from API)."),
        choices=STATUS_CHOICES,
        blank=True, null=True
    )
    result = models.CharField(
        max_length=20,
        choices=RESULT_CHOICES,
        help_text=_("The final result of the check / reports (from API)."),
        blank=True, null=True
    )
    updated_at = models.DateTimeField(
        blank=True, null=True,
        help_text=_("The timestamp of the most recent status change (from API).")
    )

    class Meta:
        abstract = True

    def update_status(self, event, new_status, timestamp):
        """
        Update the status field of the object and fire signal(s).

        When the app receives an event callback from Onfido, we update
        the status of the relevant Check/Report, and then fire the
        signals that allow external apps to hook in to this event.

        If the update is a change to 'complete', then we fire a second
        signal - 'complete' is the terminal state change, and therefore
        of most interest to clients - typically the on_status_update
        signal would be registered for logging a complete history of
        changes, whereas the on_completion signal would be used to do
        something more useful - updating the status of the user, sending
        them an email etc.

        Args:
            event: string, the name of the event, e.g. 'report.completed'
            new_status: string, the new status, e.g. 'complete'
            timestamp: DateTime, from the callback `completed_at` value

        Returns the updated object.

        """
        # we're doing a lot of marshalling from JSON to python, so this assert
        # just ensures we do actually have a datetime at this point
        assert isinstance(timestamp, datetime.datetime)
        # swap statuses around so we record old / new
        self.status, old_status = new_status, self.status
        self.updated_at = timestamp
        self.save()
        on_status_change.send(
            self.__class__,
            instance=self,
            event=event,
            status_before=old_status,
            status_after=new_status
        )
        if new_status == 'complete':
            on_completion.send(self.__class__, instance=self)
        return self

    def parse(self, raw_json):
        """Parses the raw value out into other properties."""
        super(BaseStatusModel, self).parse(raw_json)
        self.result = self.raw['result']
        self.status = self.raw['status']
        return self


class ApplicantManager(models.Manager):

    """Custom Applicant queryset."""

    def create_applicant(self, user, raw):
        """Create a new applicant in Onfido from a user."""
        logger.debug("Creating new Onfido applicant from JSON: %s", raw)
        return Applicant(user=user).parse(raw).save()


class Applicant(BaseModel):

    """An Onfido applicant record."""

    user = models.OneToOneField(
        User,
        help_text=_("Django user that maps to this applicant."),
        related_name='onfido_applicant'
    )

    objects = ApplicantManager()

    def __unicode__(self):
        return self.user.get_full_name() or self.user.username

    def __repr__(self):
        return u"<Applicant id=%s user='%s'>" % (
            self.id, self.user.username
        )


class CheckManager(models.Manager):

    """Check model manager."""

    def create_check(self, applicant, raw):
        """Create a new Check object from the raw JSON."""
        logger.debug("Creating new Onfido check from JSON: %s", raw)
        return Check(user=applicant.user, applicant=applicant).parse(raw).save()


class Check(BaseStatusModel):

    """The state of an individual check made against an Applicant."""

    CHECK_TYPE_CHOICES = (
        ('express', 'Express check'),
        ('standard', 'Standard check')
    )

    user = models.ForeignKey(
        User,
        help_text=_("The Django user (denormalised from Applicant to make navigation easier)."),  # noqa
        related_name='onfido_checks'
    )
    applicant = models.ForeignKey(
        Applicant,
        help_text=_("The applicant for whom the check is being made."),
        related_name='checks'
    )
    check_type = models.CharField(
        max_length=10,
        choices=CHECK_TYPE_CHOICES,
        help_text=_("See https://documentation.onfido.com/#check-types")
    )

    objects = CheckManager()

    def __unicode__(self):
        return (
            u"%s for %s" % (
                self.get_check_type_display().capitalize(),
                self.user.get_full_name() or self.user.username
            )
        )

    def __repr__(self):
        return (
            u"<Check id=%s type='%s' user='%s'>" % (
                self.id,
                self.check_type,
                self.user
            )
        )

    def parse(self, raw_json):
        """Parses the raw value out into other properties."""
        super(Check, self).parse(raw_json)
        self.check_type = self.raw['type']
        return self


class ReportManager(models.Manager):

    """Report model manager."""

    def create_report(self, check, raw):
        """Create a new Report from the raw JSON."""
        logger.debug("Creating new Onfido report from JSON: %s", raw)
        return Report(user=check.user, onfido_check=check).parse(raw).save()


class Report(BaseStatusModel):

    """Specific reports associated with a Check."""

    REPORT_TYPE_CHOICES = (
        ('identity', 'Identity report'),
        ('document', 'Document report'),
        ('street_level', 'Street level report'),
        ('facial_similarity', 'Facial similarity report'),
        ('credit', 'Credit report'),
        ('criminal_history', 'Criminal history'),
        ('right_to_work', 'Right to work'),
        ('ssn_trace', 'SSN trace'),
    )

    user = models.ForeignKey(
        User,
        help_text=_("The Django user (denormalised from Applicant to make navigation easier)."),  # noqa
        related_name='onfido_reports'
    )
    onfido_check = models.ForeignKey(
        Check,
        help_text=_("Check to which this report is attached."),
        related_name='reports'
    )
    report_type = models.CharField(
        max_length=20,
        choices=REPORT_TYPE_CHOICES,
        help_text=_("The name of the report - see https://documentation.onfido.com/#reports")
    )

    objects = ReportManager()

    def __unicode__(self):
        return (
            u"%s for %s" % (
                self.get_report_type_display().capitalize(),
                self.user.get_full_name() or self.user.username
            )
        )

    def __repr__(self):
        return (
            u'<Report id=%s type=%s user=%s>' % (
                self.id,
                self.report_type,
                self.user
            )
        )

    def parse(self, raw_json):
        """Parses the raw value out into other properties."""
        super(Report, self).parse(raw_json)
        self.report_type = self.raw['name']
        return self



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0004_auto_20161019_1259'),
    ]

    operations = [
        migrations.AlterField(
            model_name='report',
            name='report_type',
            field=models.CharField(help_text='The name of the report - see https://documentation.onfido.com/#reports', max_length=20, choices=[(b'identity', b'Identity report'), (b'document', b'Document report'), (b'street_level', b'Street level report'), (b'facial_similarity', b'Facial similarity report'), (b'credit', b'Credit report'), (b'criminal_history', b'Criminal history'), (b'right_to_work', b'Right to work'), (b'ssn_trace', b'SSN trace')]),
        ),
    ]
